/// Predictable Dart bindings for the Discord API.
library selene;

import 'dart:async';
import 'dart:convert';
import 'dart:core';
import 'dart:io';

import 'package:logging/logging.dart';
import 'package:meta/meta.dart';
import 'package:w_transport/vm.dart' show vmTransportPlatform;
import 'package:w_transport/w_transport.dart' as transport;

part 'src/core/cache.dart';
part 'src/core/client.dart';
part 'src/core/dispatcher.dart';
part 'src/models/channels/channel.dart';
part 'src/models/channels/dm_channel.dart';
part 'src/models/channels/guild_category_channel.dart';
part 'src/models/channels/guild_channel.dart';
part 'src/models/channels/guild_text_channel.dart';
part 'src/models/channels/guild_voice_channel.dart';
part 'src/models/channels/text_channel.dart';
part 'src/models/emote.dart';
part 'src/models/entity.dart';
part 'src/models/guild_role.dart';
part 'src/models/guilds/guild.dart';
part 'src/models/member.dart';
part 'src/models/message.dart';
part 'src/models/model_exception.dart';
part 'src/models/permission_overwrite.dart';
part 'src/models/reaction.dart';
part 'src/models/user.dart';
part 'src/rest/ratelimited_exception.dart';
part 'src/rest/request_bucket.dart';
part 'src/rest/rest_api.dart';
part 'src/rest/rest_api_base.dart';
part 'src/util/embed_builder.dart';
part 'src/util/images.dart';
part 'src/util/serializable.dart';
part 'src/util/snowflake.dart';
part 'src/ws/events/ready_event.dart';
part 'src/ws/events/ws_event.dart';
part 'src/ws/frames/heartbeat_frame.dart';
part 'src/ws/frames/identify_frame.dart';
part 'src/ws/frames/ws_frame.dart';
part 'src/ws/websocket.dart';
part 'src/ws/ws_base.dart';


/// The version number of this Selene version.
String versionNumber = '0.2.0';

/// Represents a WebSocket OP Code frame handler.
typedef PayloadHandler = Future Function(_WSFrame frame);

/// Removes all null records from a JSON map.
///
/// Only removes null records from the top-level.
void removeNullRecords(Map<String, dynamic> input) {
  input.removeWhere((k, v) => v == null);
}

/// Safely parses an integer.
///
/// Returns null if the deserialization fails, or the input is null.
///
/// Replaces [int.tryParse], because it does not support null inputs.
int parseInt(String input) {
  if (input == null) return null;
  return int.tryParse(input);
}

/// Returns `replacer` if `input` is null or empty.
///
/// You can use this to avoid `BAD REQUEST` crashes when sending embed fields.
String emptyReplace(String input, String replacer) {
  if (input == null || input.isEmpty) return replacer;
  else return input;
}