part of selene;

/// A utility and storage class to hold cached API objects.
class EntityCache {
  /// The [Client] that this cache is attached to.
  Client client;

  /// The current connected Discord user.
  User self;

  /// All users that this client has seen.
  Map<int, User> users = {};

  /// All guild members that this client has seen.
  Map<int, GuildMember> members = {};

  /// All guilds that this client has seen.
  Map<int, Guild> guilds = {};

  /// All channels that this client has seen.
  Map<int, GuildChannel> guildChannels = {};

  /// All DM channels that this client has sent/received.
  Map<int, DMChannel> dmChannels = {};

  /// All guild emotes that this client has seen.
  Map<int, GuildEmote> guildEmotes = {};

  /// All guild roles that this client has seen.
  Map<int, GuildRole> roles = {};

  /// Creates an empty [EntityCache].
  EntityCache(this.client);

  /// Resolves a [GuildChannel] or [DMChannel] object from the cache.
  ///
  /// Returns null if the cache record does not exist.
  Channel resolveChannel(int id) {
    if (guildChannels.containsKey(id)) {
      // Guild channel
      return guildChannels[id];
    } else if (dmChannels.containsKey(id)) {
      // DM channel
      return dmChannels[id];
    } else
      return null; // Invalid key
  }
}
