part of selene;

/// A generic channel in Discord.
abstract class Channel extends Entity {
  /// Creates a new [Channel].
  Channel(Client client) : super(client);

  /// The type of this channel.
  ChannelType type;

  @override
  void _update(Map<String, dynamic> model) {
    super._update(model);

    if (model['type'] != null) {
      switch (model['type']) {
        case 0:
          type = ChannelType.GuildText;
          break;
        case 1:
          type = ChannelType.DM;
          break;
        case 2:
          type = ChannelType.GuildVoice;
          break;
        case 3:
          type = ChannelType.GroupDM;
          break;
        case 4:
          type = ChannelType.GuildCategory;
          break;
        default:
          throw new ModelUpdateException(
              this,
              'Received invalid channel type from Discord. Is this library out-of-date?');
      }
    }
  }

  /// Generates a [Channel] based off of the supplied data.
  static Channel findImpl(Map<String, dynamic> model, Client client, [Guild guild = null]) {
    if (model['type'] == null) return null;
    switch (model['type']) {
      case 0:
        return new GuildTextChannel(guild, client);
      case 1:
        return new DMChannel(client);
      case 2:
        return new GuildVoiceChannel(guild, client);
      case 4:
        return new CategoryChannel(guild, client);
      default:
        return null;
    }
  }
}

/// A type of channel.
enum ChannelType { GuildText, GuildVoice, GuildCategory, DM, GroupDM }
