part of selene;

/// A channel belonging to a guild in Discord.
abstract class GuildChannel extends Channel {
  /// The [Guild] this channel belongs to.
  Guild guild;

  /// The parent category of this channel.
  ///
  /// Null if it does not have a parent.
  CategoryChannel get category {
    if (_parentId == null) return null;
    return guild.channels[_parentId];
  }

  int _parentId;

  /// The sorting position of this channel in the channel listing.
  int sortingPosition;

  /// The name of this channel.
  String name;

  /// The permission overwrites in this channel.
  List<PermissionOverwrite> permissionOverwrites = [];

  /// Whether this channel is NSFW.
  bool isNsfw;

  /// Whether this channel is not NSFW.
  bool get isNotNsfw => !isNsfw;

  /// Creates a new [GuildChannel].
  GuildChannel(this.guild, Client client) : super(client);

  @override
  void _update(Map<String, dynamic> model) {
    super._update(model);

    sortingPosition = model['position'] ?? sortingPosition;
    name = model['name'] ?? name;
    isNsfw = model['nsfw'] ?? isNsfw;

    _parentId = parseInt(model['parent_id']);

    if (model['permission_overwrites'] != null) {
      for (var jsonOverwrite in model['permission_overwrites']) {
        var overwrite = new PermissionOverwrite(client);
        overwrite._update(jsonOverwrite);
        permissionOverwrites.add(overwrite);
      }
    }
  }

  @override
  String toString() => "${name} (ID ${id})";
}
