part of selene;

/// Represents an emote in Discord.
///
/// This Emote can be a standard Emote or a guild Emote.
class Emote extends Entity {
  /// Creates a new [Emote].
  Emote(Client client) : super(client);

  /// The name of this emote.
  String name;

  @override
  void _update(Map<String, dynamic> model) {
    super._update(model);

    name = model['name'] ?? name;
  }

  /// Determines the type and returns a new [Emote].
  static Emote fromJson(Map<String, dynamic> model, Client client, [Guild guild = null]) {
    if (model['id'] == null) {
      return new StandardEmote(client);
    }
    return new GuildEmote(guild, client);
  }
}

/// Represents a custom emote on a [Guild].
class GuildEmote extends Emote {
  /// Creates a new [GuildEmote].
  GuildEmote(this.guild, Client client) : super(client);

  /// The guild that owns this emote.
  Guild guild;

  /// A list of roles that this emote is whitelisted to.
  List<GuildRole> roles = [];

  /// The user who created this emote.
  User author;

  /// Whether this emote must be wrapped in colons.
  bool requiresColons;

  /// Whether this emote doesn't have to be wrapped in colons.
  bool get doesNotRequireColons => !requiresColons;

  /// Whether this emote is managed by an integration.
  bool isManaged;

  /// Whether this emote is not managed by an integration.
  bool get isNotManaged => !isManaged;

  /// Whether this emote is animated.
  bool isAnimated;

  /// Whether this emote is not animated.
  bool get isNotAnimated => !isAnimated;

  @override
  void _update(Map<String, dynamic> model) {
    super._update(model);

    for (var jsonRole in model['roles'] ?? []) {
      if (guild.roles.any((r) => r.id == int.tryParse(jsonRole ?? ''))) {
        roles.add(guild.roles.firstWhere((r) => r.id == int.tryParse(jsonRole ?? '')));
      }
    }

    if (model['user'] != null) {
      var tempAuthor = new User(client);
      tempAuthor._update(model['user']);
      author = tempAuthor;
    }

    requiresColons = model['require_colons'] ?? requiresColons;
    isManaged = model['managed'] ?? isManaged;
    isAnimated = model['animated'] ?? isAnimated;
  }
}

/// Represents a standard Discord Emote.
class StandardEmote extends Emote {
  /// Creates a new [StandardEmote].
  StandardEmote(Client client) : super(client);
}
