part of selene;

/// A generic Discord entity.
abstract class Entity {
  /// The ID of this entity.
  ///
  /// This is a Snowflake entity sent and received as a 32-bit integer.
  int id;

  /// The ID of this entity as a Snowflake object.
  Snowflake get snowflakeId => new Snowflake.parseInt(id);

  /// The client that this entity is attached to.
  ///
  /// Usually this is the client that instantiated or retrieved this entity.
  Client client;

  /// Creates a new blank entity using the specified client.
  Entity(this.client);

  /// Updates this entity, setting all properties to those available in [model].
  @mustCallSuper
  void _update(Map<String, dynamic> model) {
    id = parseInt(model['id']); // set snowflake identifier for all common objects
  }

  /// Shortcut for updating this entity using a [transport.Response]'s JSON body.
  void _updateFromResponse(transport.Response response) =>
      _update(response.body.asJson());

  @override
  String toString() => "Discord entity of type '${runtimeType.toString()}' (ID ${id})";
}