part of selene;

/// A member of a guild in Discord.
class GuildMember extends Entity {
  /// The internal [User] object that this member wraps.
  User user;

  /// The guild that this member is a part of.
  Guild guild;

  /// The nickname of this member.
  String nickname;

  /// The roles that this member has.
  List<GuildRole> roles = [];

  /// When this member joined the server.
  DateTime joined;

  /// Whether this member is currently deafened.
  bool isDeafened;

  /// Whether this member is currently muted.
  bool isMuted;

  /// Creates a new [GuildMember].
  GuildMember(this.guild, Client client) : super(client);

  /// Whether this member is currently not deafened.
  bool get isNotDeafened => !isDeafened;

  /// Whether this member is currently not muted.
  bool get isNotMuted => !isMuted;

  @override
  String toString() => "${user.username}, member of ${guild.name} (ID ${id})";

  @override
  void _update(Map<String, dynamic> model, [bool isUpdating = false]) {
    super._update(model);
    var userId = int.tryParse((model['user']['id']) ?? '');
    id = userId;

    void parseRoles() {
      for (var jsonRole in model['roles'] ?? []) {
        if (this.guild != null) {
          roles.add(guild.roles.firstWhere((r) => r.id == int.tryParse(jsonRole)));
        }
      }
    }

    if (model['guild_id'] != null) {
      guild = client.cache.guilds[parseInt(model['guild_id'])];
    }

    User root;

    if (!client.cache.users.containsKey(userId)) {
      var newUser = new User(client);
      newUser._update(model['user']);
      client.cache.users[newUser.id] = newUser;
      root = newUser;
    } else {
      root = client.cache.users[id];
    }

    if (isUpdating) roles = [];
    parseRoles();

    user = root;

    nickname = model['nick']; // don't check for null because not being sent indicates no nickname
    joined = DateTime.tryParse(model['joined_at'] ?? '') ?? joined;
    isDeafened = model['deaf'] ?? isDeafened;
    isMuted = model['mute'] ?? isMuted;
  }
}
