part of selene;

class User extends Entity {
  /// Creates a new [User].
  User(Client client) : super(client);

  /// This user's username.
  ///
  /// Not unique across the Discord platform.
  String username;

  /// This user's 4-digit Discord Tag.
  String discriminator;

  /// The default avatar for this user's discriminator.
  String getDefaultAvatarUrl({String format = 'png', int size}) =>
      _ImageHelper.getDefaultAvatarUrl(
          discriminatorValue, format: format, size: size);

  /// The current avatar URL of this user.
  String getAvatarUrl([String format = 'png', int size]) =>
      _ImageHelper.getAvatarUrl(id, avatarHash, format: format, size: size);

  /// This user's 4-digit Discord Tag parsed to an integer.
  int get discriminatorValue => int.parse(discriminator);

  /// The avatar hash of this user.
  String avatarHash;

  /// Whether this user is a bot.
  bool isBot;

  /// Whether this user is not a bot.
  bool get isNotBot => !isBot;

  /// Whether this user has MFA enabled.
  bool hasMFAEnabled;

  /// Whether this user does not have MFA enabled.
  bool get doesNotHaveMFAEnabled => !hasMFAEnabled;

  @override
  void _update(Map<String, dynamic> model) {
    super._update(model);
    username = model['username'] ?? username;
    discriminator = model['discriminator'] ?? discriminator;
    avatarHash = model['avatar'] ?? avatarHash;
    isBot = model['isBot'] ?? isBot;
    if (model['isBot'] == null || model['isBot'] == false) {
      isBot = false;
    } else {
      isBot = true;
    }
    hasMFAEnabled = model['mfa_enabled'] ?? hasMFAEnabled;
  }

  String toString() => "${username} (ID ${id})";
}
