part of selene;

/// The default implementation of the Discord REST API.
class _RestImpl implements RestBase {
  transport.HttpClient client;

  String token;

  String type;

  Uri baseUri = Uri.parse('https://discordapp.com/api/v6');

  _RestImpl() {
    client = new transport.HttpClient(transportPlatform: vmTransportPlatform);
  }

  void login(String token, {String type = 'Bot'}) {
    this.token = token;
    this.type = type;
  }

  Future<transport.Response> makeNewRequest(String uriReference, String method,
      [Map<String, dynamic> params, bool useBucket = true]) async {
    var request = new transport.JsonRequest(
        transportPlatform: vmTransportPlatform);
    request.uri = Uri.parse(baseUri.toString() + uriReference);

    request.headers['Authorization'] = (type ?? '') + ' ' + (token ?? '');
    request.headers['User-Agent'] =
    'DiscordBot (https://github.com/sarcasmloading/selene, $versionNumber)';
    if (params != null) request.body = params;

    if (!useBucket) return await request.send(method);

    var bucket = new RequestBucket.getOrCreate(request.uri, this);
    return await bucket.executeRequest(request, method);
  }

  @override
  Future<transport.Response> addPinnedChannelMessage(int channelId,
      int messageId) =>
      makeNewRequest('/channels/$channelId/pins/$messageId', 'PUT');

  @override
  Future<transport.Response> addRecipientToGroupDm(int channelId, int userId,
      Map<String, dynamic> params) =>
      makeNewRequest('/channels/$channelId/recipients/$userId', 'PUT', params);

  @override
  Future<transport.Response> createChannelInvite(int channelId,
      Map<String, dynamic> params) =>
      makeNewRequest('/channels/$channelId/invites', 'POST', params);

  @override
  Future<transport.Response> createMessage(int channelId,
      Map<String, dynamic> params) =>
      makeNewRequest('/channels/$channelId/messages', 'POST', params);

  @override
  Future<transport.Response> createReaction(int channelId, int messageId,
      String emoteFormat) =>
      makeNewRequest(
          '/channels/$channelId/messages/$messageId/reactions/$emoteFormat/@me',
          'PUT');

  @override
  Future<transport.Response> deleteAllReactions(int channelId, int messageId) =>
      makeNewRequest(
          '/channels/$channelId/messages/$messageId/reactions', 'DELETE');

  @override
  Future<transport.Response> deleteChannel(int id) =>
      makeNewRequest('/channels/$id', 'DELETE');

  @override
  Future<transport.Response> deleteChannelPermission(int channelId,
      int overwriteId) =>
      makeNewRequest('/channels/$channelId/permissions/$overwriteId', 'DELETE');

  @override
  Future<transport.Response> deleteMessage(int channelId, int messageId) =>
      makeNewRequest('/channels/$channelId/messages/$messageId', 'DELETE');

  @override
  Future<transport.Response> deleteMessagesBulk(int channelId,
      Map<String, dynamic> params) =>
      makeNewRequest(
          '/channels/$channelId/messages/bulk-delete', 'DELETE', params);

  @override
  Future<transport.Response> deleteOwnReaction(int channelId, int messageId,
      String emoteFormat) =>
      makeNewRequest(
          '/channels/$channelId/messages/$messageId/reactions/$emoteFormat/@me',
          'DELETE');

  @override
  Future<transport.Response> deletePinnedChannelMessage(int channelId,
      int messageId) =>
      makeNewRequest('/channels/$channelId/pins/$messageId', 'DELETE');

  @override
  Future<transport.Response> deleteUserReaction(int channelId,
      int messageId, int EmoteFormat, int userId) =>
      makeNewRequest(
          '/channels/$channelId/messages/$messageId/reactions/$EmoteFormat/$userId',
          'DELETE');

  @override
  Future<transport.Response> editChannelPermissions(int channelId,
      int overwriteId, Map<String, dynamic> params) =>
      makeNewRequest(
          '/channels/$channelId/permissions/$overwriteId', 'PUT', params);

  @override
  Future<transport.Response> editMessage(int channelId, int messageId,
          [Map<String, dynamic> params]) =>
      makeNewRequest(
          '/channels/$channelId/messages/$messageId', 'PATCH', params);

  @override
  Future<transport.Response> getChannel(int id) =>
      makeNewRequest(
        '/channels/$id',
        'GET',
      );

  @override
  Future<transport.Response> getChannelInvites(int channelId) =>
      makeNewRequest('/channels/$channelId/invites', 'GET');

  @override
  Future<transport.Response> getChannelMessage(int channelId, int messageId) =>
      makeNewRequest('/channels/$channelId/messages/$messageId', 'GET');

  @override
  Future<transport.Response> getChannelMessages(int channelId,
          [Map<String, dynamic> params]) =>
      makeNewRequest('/channels/$channelId/messages', 'GET');

  @override
  Future<transport.Response> getPinnedMessages(int channelId) =>
      makeNewRequest('/channels/$channelId/pins', 'GET');

  @override
  Future<transport.Response> getReactions(int channelId, int messageId,
      int EmoteFormat,
          [Map<String, dynamic> params]) =>
      makeNewRequest(
          '/channels/$channelId/messages/$messageId/reactions/$EmoteFormat',
          'GET', params);

  @override
  Future<transport.Response> modifyChannel(int id, String method,
          [Map<String, dynamic> params]) =>
      makeNewRequest('/channels/$id', method, params);

  @override
  Future<transport.Response> removeRecipientFromGroupDm(int channelId,
      int userId) =>
      makeNewRequest('/channels/$channelId/recipients/$userId', 'DELETE');

  @override
  Future<transport.Response> triggerTypingIndicator(int channelId) =>
      makeNewRequest('/channels/$channelId/typing', 'POST');

  @override
  Future<transport.Response> getGateway() =>
      makeNewRequest('/gateway', 'GET', null, false);

  @override
  Future<transport.Response> getGatewayWithShards() =>
      makeNewRequest('/gateway/bot', 'GET', null, false);
}
