part of selene;

/// An event invoked when the client receives the `READY` event.
class ReadyEvent extends _WSEvent {
  /// Creates a new [ReadyEvent].
  ReadyEvent(Client client) : super(client);

  /// The current user.
  User get self => client.cache.self;

  /// The guilds (lazily initialized) that this user is a member of.
  List<Guild> get guilds => client.cache.guilds.values;

  @override
  Future onReceive(Map<String, dynamic> data) async {
    if (data['guilds'] != null) {
      await Future.forEach(data['guilds'], (lazyJsonGuild) async {
        var lazyGuild = new Guild(client);
        lazyGuild._update(lazyJsonGuild);
        client.cache.guilds[lazyGuild.id] = lazyGuild;
      });
    }
    if (data['user'] != null) {
      var newUser = new User(client);
      newUser._update(data['user']);
      client.cache.self = newUser;
      client.cache.users[newUser.id] = newUser; // Cache ourselves
    }
    if (data['private_channels'] != null) {
      await Future.forEach(data['private_channels'],
          (privateChannelJson) async {
        var privateChannel = new DMChannel(client);
        privateChannel._update(privateChannelJson);
        client.cache.dmChannels[privateChannel.id] = privateChannel;
      });
    }
  }
}
