import 'dart:async';
import 'dart:io';

import 'package:logging/logging.dart';
import 'package:selene/selene.dart';

void main() {
  devBot();
}

Future devBot() async {
  var client = new Client();
  Logger.root.onRecord.listen((LogRecord record) {
    print('[${record.loggerName}] ${record.level.name}: ${record.time}: ${record.message}');
  });

  client.dispatcher.onMessageReceive.listen((Message msg) async {
    if (msg.content == '!cache') {
      if (msg.channel is GuildChannel) {
        var guildChannel = msg.channel as GuildTextChannel;
        print('Getting cache for guild ' + guildChannel.guild.name);
        var guild = guildChannel.guild;
        var sb = new StringBuffer();
        var gc = guild.channels.values;
        await Future.forEach(gc,
            (GuildChannel guildChannel) async {
          if (guildChannel is CategoryChannel) {
            sb.writeln('Category ${guildChannel.name}');
            await Future.forEach(guildChannel.channels, (GuildChannel child) {
              sb.writeln(
                  '      ${child} (`${child.runtimeType}`)');
            });
          } else if (guildChannel.category == null) {
            sb.writeln('${guildChannel}');
          }
        });
        await guildChannel.sendMessage(
            content: (sb.toString().isEmpty ? 'Nothing.' : sb.toString()));
      } else if (msg.channel is DMChannel) {
        await msg.channel.sendMessage(content: 'Hello DM!');
      }
    }
    if (msg.content == '!delete') {
      var newMsg = await msg.channel.sendMessage(
          content: "I am going to delete this message in 3 seconds.");
      await new Future.delayed(new Duration(seconds: 3), () async {
        await newMsg.delete();
      });
    }
    if (msg.content == '!edit') {
      var oldMsg = await msg.channel.sendMessage(
          content: "This is the old message. Updating in 3 seconds.");
      await new Future.delayed(new Duration(seconds: 3), () async {
        await oldMsg.edit((x) {
          x.content = "This is the new message.";
        });
      });
    }
    if (msg.content == '!self') {
      var self = client.cache.self;
      var embed = <String, dynamic>{
        'author': {
          'name': self.username,
          'icon_url': self.getAvatarUrl()
        },
        'description': 'I am the Selene developer test bot.'
      };
      await msg.channel.sendMessage(embed: embed);
    }
    if (msg.content == '!guild') {
      var message = msg.channel as GuildTextChannel;
      var guild = message.guild;
      var embed = {
        'title': 'Guild ${guild.name}',
        'description': 'Guild properties overview',
        'fields': [
          {
            'name': 'ID',
            'value': guild.id ?? 'blank',
            'inline': true
          },
          {
            'name': 'Channel Length',
            'value': guild.channels.length,
            'inline': true
          },
          {
            'name': 'AFK Channel ID',
            'value': guild.afkChannelId ?? 'none',
            'inline': true
          },
          {
            'name': 'AFK Timeout',
            'value': guild.afkTimeout ?? 'none',
            'inline': true
          },
          {
            'name': 'Roles',
            'value': (guild.roles ?? []).map((GuildRole role) => role.name).join(
                ", "),
            'inline': true
          },
          {
            'name': 'Voice Region',
            'value': guild.voiceRegion,
            'inline': true
          }
        ]
      };

      await msg.channel.sendMessage(embed: embed);
    }
    if (msg.content == '!ratelimit') {
      // Attempt to invoke message send rate limit
      await Future.forEach(new Iterable.generate(15, (ix) {
        return ix;
      }), (i) async {
        await msg.channel.sendMessage(content: "Rate limiting - message ${i}");
      });
    }
    if (msg.content == '!snowflake') {
      var snowflake = msg.snowflakeId;
      await msg.channel.sendMessage(
          content: 'Your message was sent at ${snowflake.timestamp
              .toString()}, on the worker ${snowflake
              .workerId}, on process ${snowflake
              .processId}, with an internal ID of ${snowflake.idOnProcess}.');
    }
    if (msg.content == '!member') {
      var member = msg.member;
      var embed = <String, dynamic>{
        'title': 'Info for ${member.nickname ?? member.user.username}#${member.user.discriminator}',
        'description': 'Has ${member.roles.length} roles',
        'fields': [
          {
            'name': 'Roles',
            'value': emptyReplace(member.roles.map((role) => role.name).join(", "), 'None'),
            'inline': true,
          },
          {
            'name': 'Deafened/Muted',
            'value': '${member.isDeafened}/${member.isMuted}',
            'inline': true
          },
          {
            'name': 'Has Nickname',
            'value': '${member.nickname != null ? 'Yes (${member.nickname})' : 'No'}.',
            'inline': true
          }
        ]
      };
      await msg.channel.sendMessage(embed: embed);
    }
    if (msg.content == '!category') {
      await msg.channel.sendMessage(content: 'This channel\'s category is ${(msg.channel as GuildTextChannel).category.name}.');
    }
    if (msg.content == '!channel') {
      if (msg.channel.type == ChannelType.DM) {
        var dm = msg.channel as DMChannel;
        await msg.channel.sendMessage(content: dm.toString());
        return;
      }

      var channel = msg.channel as GuildTextChannel;
      var embed = <String, dynamic>{
        'title': 'Info for channel ${channel.name}',
        'description': 'Discord channel information',
        'fields': [
          {
            'name': 'Topic',
            'value': channel.topic ?? 'No topic',
            'inline': true
          }
        ]
      };

      await channel.sendMessage(embed: embed);
    }
    if (msg.content == '!dumpall') {
      String dumpUsers() {
        return "**Users**\n" +
            client.cache.users.values.map((User user) => user.toString()).join(
                "\n");
      }
      String dumpGuilds() {
        return "**Guilds**\n" +
            client.cache.guilds.values.map((Guild guild) => guild.toString())
                .join("\n");
      }
      String dumpDms() {
        return "**DMs**\n" +
            client.cache.dmChannels.values.map((DMChannel channel) =>
                channel.toString()).join("\n");
      }
      String dumpMembers() {
        return "**Members**\n" +
            client.cache.members.values.map((GuildMember member) =>
                member.toString()).join("\n");
      }
      await msg.channel.sendMessage(
          content: dumpUsers() + "\n\n" + dumpGuilds() + "\n\n" + dumpDms() +
              "\n\n" + dumpMembers());
    }
    if (msg.content == '!members') {
      await msg.channel.sendMessage(
          content: (msg.channel as GuildTextChannel).guild.members.values.map((member) => member.user.username).join(", "));
    }
    if (msg.content == '!messages') {
      var messageString = msg.channel.messages
          .map((id, msg) =>
      new MapEntry(id, "${msg.author.username}: ${msg.content}"))
          .values
          .join("\n");
      await msg.channel.sendMessage(content: messageString);
    }
  });
  client.dispatcher.onReady.listen((_) {
    new Logger('Selene-Test').info('Client is ready');
  });

  await client.start();
  await client.login(Platform.environment['SELENE_TOKEN']);
}
